# Generated by Django 4.1.7 on 2023-08-16 05:08

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import shortuuid.main


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('nets_core', '0002_alter_verificationcode_token'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserDevice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('name', models.CharField(max_length=250, verbose_name='Name')),
                ('os', models.CharField(blank=True, max_length=250, null=True, verbose_name='OS')),
                ('os_version', models.CharField(blank=True, max_length=250, null=True, verbose_name='OS Version')),
                ('app_version', models.CharField(blank=True, max_length=250, null=True, verbose_name='App Version')),
                ('device_id', models.CharField(default=shortuuid.main.ShortUUID.uuid, max_length=250, verbose_name='Device ID')),
                ('device_type', models.CharField(blank=True, max_length=250, null=True, verbose_name='Device Type')),
                ('device_token', models.CharField(blank=True, max_length=250, null=True, verbose_name='Device Token')),
                ('firebase_token', models.CharField(blank=True, max_length=250, null=True, verbose_name='Firebase Token')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('last_login', models.DateTimeField(null=True, verbose_name='Last login')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User Device',
                'verbose_name_plural': 'User Devices',
                'db_table': 'nets_core_user_device',
            },
        ),
    ]
